function exportValuesFromGroups(aImarisApplicationID)
% Export spots to matlab
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="ANALYZE WAVE">
%        <Item name="Export Spots Intensities" icon="Matlab" tooltip="Export Spots Intensities">
%          <Command>MatlabXT::exportValuesFromGroups(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%    </CustomTools>
%
% Copyright (c) 2016, Dominic Filion

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
    javaaddpath ImarisLib.jar
    vImarisLib = ImarisLib;
    if ischar(aImarisApplicationID)
        aImarisApplicationID = round(str2double(aImarisApplicationID));
    end
    vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
    vImarisApplication = aImarisApplicationID;
end

% the user has to create a scene with some spots
vSurpassScene = vImarisApplication.GetSurpassScene;
if isequal(vSurpassScene, [])
    msgbox('Please create some Spots in the Surpass scene!');
    return;
end

% get the spots
vSpots = vImarisApplication.GetSurpassSelection;
vSpotsSelected = vImarisApplication.GetFactory.IsSpots(vSpots);

if vSpotsSelected
    vScene = vSpots.GetParent;
else
    vScene = vImarisApplication.GetSurpassScene;
end
vNumberOfSpots = 0;

% find datacontainer of children
for vChildIndex = 1:vScene.GetNumberOfChildren
    vDataItem = vScene.GetChild(vChildIndex - 1);
    % check if dataitem is a Data container
    if vImarisApplication.GetFactory.IsDataContainer(vDataItem)
        vDataContainer = vImarisApplication.GetFactory.ToDataContainer(vDataItem);
        % check all items in the data container
        for vNewChildIndex = 1:vDataContainer.GetNumberOfChildren
            vNewDataItem = vDataContainer.GetChild(vNewChildIndex - 1);
            if vNewDataItem.GetVisible()
                if vImarisApplication.GetFactory.IsSpots(vNewDataItem)
                    vNumberOfSpots = vNumberOfSpots+1;
                    vSpotsList{vNumberOfSpots} = vImarisApplication.GetFactory.ToSpots(vNewDataItem); %#ok<AGROW>
                    vNamesList{vNumberOfSpots} = char(vNewDataItem.GetName); %#ok<AGROW>
                end
            end
        end
        % if it is not a data container, it may be a spots
    else
        if vDataItem.GetVisible()
            if vImarisApplication.GetFactory.IsSpots(vDataItem)
                vNumberOfSpots = vNumberOfSpots+1;
                vSpotsList{vNumberOfSpots} = vImarisApplication.GetFactory.ToSpots(vDataItem);
                vNamesList{vNumberOfSpots} = char(vDataItem.GetName);
            end
        end
    end
end

if vNumberOfSpots<2
    msgbox('Please create at least 2 spots objects!');
    return;
end

vNamesList = vNamesList(1:vNumberOfSpots);

% Get file path and name

cp = cd();
    [filename,path] = uiputfile('*.mat','Please enter Filename for spots values');
cd(cp);

    % Get the spot coordinates, time indices and radius
figure,
hold on

for ii = 1:size(vNamesList,2)
    vSpots = vSpotsList{ii};
    
    spotValues = vSpots.Get();
    aXYZ = spotValues.mPositionsXYZ;
    aTime = vSpots.GetIndicesT + 1;
    radii = spotValues.mRadii;
    
    % Get the tracks and Statistics Values
    tId = vSpots.GetTrackIds;
    tEd = vSpots.GetTrackEdges;
    vStatisticValues = vSpots.GetStatistics;
    vNames = cell(vStatisticValues.mNames);
    
    % figure,
    % uicontrol('Style','listbox','Unit','normalized','Position',[0.1 0.1 .8 .8],'String',char(unique(vNames)));
    ind = strcmp(vNames,'Intensity Mean');
    vValues = vStatisticValues.mValues;
    aId = vStatisticValues.mIds;
    aId = aId(ind);
    aInt = vValues(ind);
    % Sort the values relative to Index value
    [~,ind] = sort(aId);
    aInt = aInt(ind);
    
    
    % select the group
    ids = unique(tId);
    timeMaxIncrease = zeros(size(ids,1),1);
    spotGroupTime = zeros(size(aInt,1),1);
    values = nan(size(ids,1),max(aTime)+1);
    distances = radii; %#ok<NASGU>
    valuesDiff = nan(size(ids,1),max(aTime));
    xy = nan(size(ids,1),max(aTime),2);
    for i = 1:size(ids,1)
        rEd = tEd(tId == ids(i),:);
        rS = unique(rEd);
        rT = aTime(rS+1);
        rI = aInt(rS+1);
        rTd = rT(2:end);
        [~,maxIdxDiffrI] = max(diff(rI));
        timeMaxIncrease(i) = rTd(maxIdxDiffrI);
        spotGroupTime(rS+1) = rTd(maxIdxDiffrI);
        values(i,rT+1) = rI;
        xy(i,rT+1,:) = aXYZ(rS+1,1:2);
        valuesDiff(i,rT(2:end)+1) = diff(rI);
        values(i,1) = rTd(maxIdxDiffrI);
        valuesDiff(i,1) = rTd(maxIdxDiffrI);
    end

    
    % % save the values to excel file
    %
    % vFileName = vImarisApplication.GetCurrentFileName;
    % nname = strrep(char(vFileName),'.ims','_Intensities.xlsx');
    % xlswrite(nname,values);
    % nname = strrep(char(vFileName),'.ims','_Derivatives.xlsx');
    % xlswrite(nname,valuesDiff);
    %
    %
    % timeFrame = unique(timeMaxIncrease);
    % occ = zeros(size(timeFrame,1),1);
    % for i = 1:size(occ,1)
    %     occ(i) = sum(timeMaxIncrease==timeFrame(i));
    % end
    % [~,indocc] = sort(occ,'descend');
    % % timeFrame = timeFrame(indocc);
    %
    % % Create the new spots from Group1
    % % create new group
    % vSpotsGroup = vImarisApplication.GetFactory.CreateDataContainer;
    % vSpotsGroup.SetName(sprintf('%s Groups', char(vSpots.GetName)));
    %
    % for j = 1:size(timeFrame,1)
    %     selIdx = (spotGroupTime==timeFrame(j))&(aTime>timeFrame(j)-1);
    %     vSpotsXYZ = aXYZ(selIdx,:);
    %     vSpotsTime = aTime(selIdx) - 1;
    %     vRadii = aRadii(selIdx);
    %     vNewSpots = vImarisApplication.GetFactory.CreateSpots;
    %     vNewSpots.Set(vSpotsXYZ, vSpotsTime, vRadii);
    %     % Give a color contrasted with the original color
    %     if find(indocc==j) <= 5
    %         idxColor = find(indocc==j);
    %         vNewSpots.SetColorRGBA(color(:,idxColor)); %#ok<FNDSB>
    %     else
    %         vNewSpots.SetColorRGBA(floor((256^3)*rand));
    %     end
    %     % set the cluster name and write the average spots per timepoint
    %     vNewSpots.SetName(sprintf('Group t%d (%d)',timeFrame(j),occ(j)));
    %     vSpotsGroup.AddChild(vNewSpots, -1);
    % end
    %
    %
    % % add the spots group
    % vSurpassScene.AddChild(vSpotsGroup, -1);
    
    
    % nname = strrep(char(vFileName),'.ims','.mat');
    % save(nname,'aInt','aTime','tId','tEd','values');
    
    if path ~= 0
        cd(path)
        nname = strrep(filename,'.mat',sprintf('%02d.mat',ii));
        save(nname,'aInt','aTime','tId','tEd','values','xy','distances');
        cd(cp)
    end
    plot(mean(values,'omitnan'))
    
end
%
% --------------------------------------------------------------------------
%     function [R,G,B,A] = rgba2rgb(rgba)
%
%         if rgba < 0
%             rgba = rgba + 2^32;
%         end
%         str = dec2bin(rgba);
%         strz = sprintf('%032d',0);
%         strz(end-length(str)+1:end) = str;
%         A = bin2dec(strz(1:8));
%         B = bin2dec(strz(9:16));
%         G = bin2dec(strz(17:24));
%         R = bin2dec(strz(25:32));
%     end
%
% --------------------------------------------------------------------------
%     function rgba = rgb2rgba(R,G,B)
%
%         rgba = bin2dec(sprintf('%08d%08d%08d',str2double(dec2bin(B)),...
%             str2double(dec2bin(G)),str2double(dec2bin(R))));
%     end


end